// The MIT License

// Copyright (c) 2009 Massachusetts Institute of Technology

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:

// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.

// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

import Connectable::*;
import GetPut::*;
import ClientServer::*;
import Processor::*;
import MemArb::*;
import MemTypes::*;
import FIFO::*;

//AWB includes
`include "asim/provides/low_level_platform_interface.bsh"
`include "asim/provides/soft_connections.bsh"
`include "asim/provides/common_services.bsh"

// Local includes
`include "asim/provides/processor_library.bsh"
`include "asim/provides/mem_arb.bsh"
`include "asim/provides/instruction_cache.bsh"
`include "asim/provides/data_cache.bsh"
`include "asim/provides/processor.bsh"
`include "asim/provides/audio_pipe_types.bsh"

// Scratchpad includes
`include "asim/provides/scratchpad_memory.bsh"
`include "asim/provides/mem_services.bsh"
`include "asim/dict/VDEV_SCRATCH.bsh"

interface Core;

   interface Get#(AudioProcessorUnit) sampleOutput;
   interface Put#(AudioProcessorUnit) sampleInput;
	 
//   interface CPUToHost tohost;
   
endinterface

module   [CONNECTED_MODULE]  mkCore#(Integer prog) ( Core );
   

  // Instantiate the modules

  Proc proc <- mkProc();
  ICache#(InstReq,InstResp) icache <- mkInstCache();
  DCache#(DataReq,DataResp) dcache <- mkDataCache();
  MemArb marb <- mkMemArb();
  MEMORY_IFC#(Bit#(18), Bit#(32)) memory <- mkScratchpad(prog, SCRATCHPAD_CACHED);   //Services Memory items

  // Make this big enough so that several outstanding requests may be supported
  FIFO#(Bit#(MainMemTagSz)) tags <- mkSizedFIFO(8);
   
  // Internal connections

  mkConnection( proc.statsEn_get,   icache.statsEn_put );
  mkConnection( proc.statsEn_get,   dcache.statsEn_put );
  mkConnection( proc.imem_client,   icache.proc_server );
  mkConnection( proc.dmem_client,   dcache.proc_server );
  mkConnection( icache.mmem_client, marb.cache0_server );
  mkConnection( dcache.mmem_client, marb.cache1_server );

  // Memory Access
  rule sendMemReq;
    let coreReq <- marb.mmem_client.request.get;
    case (coreReq) matches 
      tagged LoadReq .load: begin
//	                      $display("PIPE Load Addr Req %h", load.addr); 
                              //Allocate ROB space
                              memory.readReq(truncate(load.addr>>2));
                              tags.enq(load.tag);
                            end
      tagged StoreReq .store: begin
//	  	              $display("PIPE Write Addr Req %h", store.addr); 
	                      memory.write(truncate(store.addr>>2),store.data);
                              end
    endcase
  endrule
  
  rule receiveMemResp;
    let memResp <- memory.readRsp();
    tags.deq;
    marb.mmem_client.response.put(tagged LoadResp {data:memResp,
                                                   tag: tags.first});
//   	                $display("PIPE Receive MemReq %x", memResp); 
  endrule

  // Methods
     
   interface sampleOutput = proc.sampleOutput;
   interface sampleInput = proc.sampleInput;

//   interface CPUToHost tohost = proc.tohost;

endmodule
